package handler

import (
	"log/slog"
	"net/http"
	"strconv"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

func HandleTodoDelete(logger *slog.Logger, todoService TodoDeleter) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		ctx := r.Context()
		idString := r.PathValue("id")

		id, err := strconv.ParseInt(idString, 10, 64)

		if err != nil {
			slog.ErrorContext(ctx, err.Error())
			http.Error(w, "", http.StatusBadRequest)
			return
		}

		err = todoService.DeleteTodo(ctx, id)

		if err != nil {
			if err == service.ErrNotFound {
				http.Error(w, "", http.StatusNotFound)
				return
			}

			slog.ErrorContext(ctx, err.Error())
			http.Error(w, "", http.StatusInternalServerError)
			return
		}

		w.WriteHeader(http.StatusNoContent)
	}
}
